/*
 * Fonomo video chat launcher 
 * Copyright (C) 2008 Eric Windisch
 *
 * based on code by...
 * Copyright (C) 2008 Etan Reisner <deryni@pidgin.im>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301, USA.
 */

#define PURPLE_PLUGINS

#include <glib.h>


#ifndef G_GNUC_NULL_TERMINATED
# if __GNUC__ >= 4
#    define G_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
# else
#    define G_GNUC_NULL_TERMINATED
# endif
#endif

#include "notify.h"
#include "plugin.h"
#include "version.h"

#include "pidgin.h"

#include "gtkconv.h"
#include "gtkplugin.h"

#include "conversation.h"
#include "gtkconvwin.h"

#include "gtkimhtml.h"

#ifdef G_OS_WIN32
#include "win32/win32dep.h"
#endif

static void
send_button_cb(GtkButton *button, PidginConversation *gtkconv)
{
    guint32 randnum;
    gchar *url;
    gchar *url2;
    gchar *joinstr;

    randnum = g_random_int();

    url = g_strdup_printf ("http://biglinux.com.br/fonomo.php?n=1&key=%X", randnum);
    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, url);

    url2 = g_strdup_printf ("http://biglinux.com.br/fonomo.php?key=%X", randnum);
    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, url2);

    purple_notify_uri(NULL, url);
    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "url launched");

    joinstr = g_strdup_printf("Convite para uma conversa de video <BR> Invitation to a video chat <a href=\"%s\">Click aqui para iniciar!  <BR> Click here to start</a>", url2);

    gtk_imhtml_append_text(GTK_IMHTML(gtkconv->entry), joinstr, FALSE);
    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "imhtml display fonomo\n");

    g_signal_emit_by_name(gtkconv->entry, "message_send");
    g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "message sent\n");

    g_free(url);
    g_free(url2);
    g_free(joinstr);
}

static void
create_fonomo_button_pidgin (PidginConversation *conv)
{
    GtkWidget *button, *image, *sep;
    GtkIconInfo *icoinfo;
    gchar *iconames[]={"camera-web", "camera-fonomo"};

    button = gtk_button_new();
    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
    g_signal_connect(G_OBJECT(button), "clicked",
	G_CALLBACK(send_button_cb), conv);

#ifdef DATADIR
    /*
     * I prefer that we do this themable, but if we are building directly
     * against the pidgin source tree, then we will use the DATADIR
     * since that is how pidgin itself uses its icons.  This also
     * works reliably under MS Windows.
     */
    image = gtk_image_new_from_file(g_build_filename(DATADIR,"pixmaps","hicolor","22x22","devices","camera-fonomo.png",NULL));
#else
    /* We test if the current theme has an existing icon,
     * or if hicolor has an icon, and if not, 
     * we fallback to our own ugly icon.
     */
    icoinfo=gtk_icon_theme_choose_icon(
	gtk_icon_theme_get_default(),iconames,22,0);
    image = gtk_image_new_from_file(gtk_icon_info_get_filename(icoinfo));
#endif 

    gtk_container_add((GtkContainer *)button, image);

    sep = gtk_vseparator_new();

    gtk_widget_show(sep);
    gtk_widget_show(image);
    gtk_widget_show(button);

    gtk_box_pack_start(GTK_BOX(conv->toolbar), sep, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(conv->toolbar), button, FALSE, FALSE, 0);

    g_object_set_data(G_OBJECT(conv->toolbar), "fonomo_separator", sep);
    g_object_set_data(G_OBJECT(conv->toolbar), "fonomo_button", button);
}

static void
remove_fonomo_button_pidgin(PidginConversation *gtkconv)
{
    GtkWidget *send_button = NULL, *separator = NULL;

    send_button = g_object_get_data(G_OBJECT(gtkconv->toolbar), "fonomo_button");
    if (send_button != NULL) {
        gtk_container_remove(GTK_CONTAINER(gtkconv->toolbar), send_button);
        g_signal_handlers_disconnect_by_func(G_OBJECT(send_button),
                G_CALLBACK(send_button_cb), gtkconv);
        gtk_widget_destroy(send_button);
    }

    separator = g_object_get_data(G_OBJECT(gtkconv->toolbar), "fonomo_separator");
    if (send_button != NULL) {
        gtk_container_remove(GTK_CONTAINER(gtkconv->toolbar), separator);
        gtk_widget_destroy(separator);
    }
}

static gboolean
plugin_load(PurplePlugin *plugin)
{
    GList *convs = purple_get_conversations();
    void *gtk_conv_handle = pidgin_conversations_get_handle();

    purple_signal_connect(gtk_conv_handle, "conversation-displayed",
	plugin, PURPLE_CALLBACK(create_fonomo_button_pidgin), NULL);

    while (convs) {
        PurpleConversation *conv = (PurpleConversation *)convs->data;

        /* Setup Send button */
        if (PIDGIN_IS_PIDGIN_CONVERSATION(conv)) {
            create_fonomo_button_pidgin(PIDGIN_CONVERSATION(conv));
        }

        convs = convs->next;
    }

    return TRUE;
}

static gboolean
plugin_unload(PurplePlugin *plugin)
{
    GList *convs = purple_get_conversations();

    while (convs) {
        PurpleConversation *conv = (PurpleConversation *)convs->data;

        /* Remove Send button */
        if (PIDGIN_IS_PIDGIN_CONVERSATION(conv)) {
            remove_fonomo_button_pidgin(PIDGIN_CONVERSATION(conv));
        }

        convs = convs->next;
    }

    return TRUE;
}

static PurplePluginInfo info =
{
    PURPLE_PLUGIN_MAGIC,
    PURPLE_MAJOR_VERSION,                           /**< major version */
    PURPLE_MINOR_VERSION,                           /**< minor version */
    PURPLE_PLUGIN_STANDARD,                         /**< type */
    PIDGIN_PLUGIN_TYPE,                             /**< ui_requirement */
    0,                                              /**< flags */
    NULL,                                           /**< dependencies */
    PURPLE_PRIORITY_DEFAULT,                        /**< priority */
    "gtk-ewindisch-fonomobutton",                                /**< id */
    "Fonomo Video Chat",                              /**< name */
    "0.1.5",                                /**< version */
    "Spawns Fonomo chat",         /**< summary */
    "Adds a video-chat button to the "
       "the conversation window.",         /**< description */
    "Eric Windisch <eric@fonomo.com>",              /**< author */
    "http://www.fonomo.com/",                                 /**< homepage */
    plugin_load,                                    /**< load */
    plugin_unload,                                  /**< unload */
    NULL,                                           /**< destroy */
    NULL,                                           /**< ui_info */
    NULL,                                           /**< extra_info */
    NULL,                                           /**< prefs_info */
    NULL,                                           /**< actions */

    /* padding */
    NULL,
    NULL,
    NULL,
    NULL
};

static void
init_plugin(PurplePlugin *plugin)
{
}

PURPLE_INIT_PLUGIN(fonomobutton, init_plugin, info)
